from django.contrib import admin
from guardian.admin import GuardedModelAdmin
from .models import *
#import/export module
from import_export.admin import ImportExportModelAdmin

from modeltranslation.admin import TabbedTranslationAdmin
#for representative upload
from django import forms
import csv
from csv import DictReader
from django.urls import path
from django.shortcuts import redirect,render

from risefor_lobbying.functions.uploadRepresentatives import *
from risefor_lobbying.functions.updateApprovedEmails import *

import os


##########
#Representative -> Action group linking
##########
# def add_representative_to_action(modeladmin, request, queryset):
#     # do something with the queryset
#     print("hey")
# add_representative_to_action.short_description = 'Cibler '

##########
#Inline models
##########
class PhoneInline(admin.TabularInline):
    model = Phone
    extra = 1

class EmailInline(admin.TabularInline):
    model = Email
    extra = 1

class TwitterInline(admin.TabularInline):
    model = TwitterAccount
    extra = 1

class actionStepInline(admin.TabularInline):
    model = ActionStep
    extra = 1

class OrganisationLogoInline(admin.TabularInline):
    model = OrganisationLogoForAction
    extra = 1

class imageToShareInline(admin.TabularInline):
    model = imageToShare
    extra = 1



##########
#Model lists & Pages
##########

class ApprovedEmailsAdmin(admin.ModelAdmin):
    search_fields = ['name'] 
    change_list_template = './admin/entities/update-approved-emails.html'

    def get_urls(self):
        urls = super().get_urls()
        my_urls = [
            
            path('update-approved-emails/', self.update_approved_emails),
        ]
        return my_urls + urls
    def update_approved_emails(self,request):
        print("let's update the list of approved emails")
        updateApprovedEmails(self,request)

        return render(
            request, "./admin/upload-forms/start-update-emails.html"
        )
admin.site.register(ApprovedEmail, ApprovedEmailsAdmin)

#Representative PAGE & List
class RepresentativeAdmin(GuardedModelAdmin):
    ###Display & filters
    inlines = [ PhoneInline, EmailInline,TwitterInline]
    search_fields = ['fullname']
    list_filter = ('function','permanentCommission','election_department','studyGroups')
    # dropdown_filter = ('election_department') #needs specific package to enable
    list_display = ['fullname','function','politicalParty']
    # actions = [add_representative_to_action, ]

    ###csv upload function
    print("HERE",os.path.abspath(os.path.dirname(__file__)))
    change_list_template = "./admin/entities/upload-representatives-changelist.html"

    def get_urls(self):
        urls = super().get_urls()
        my_urls = [
            path('import-csv/', self.import_csv),
        ]
        return my_urls + urls

    def import_csv(self, request):
        if request.method == "POST":
            csv_file = request.FILES["csv_file"]
            decoded_file = csv_file.read().decode('utf-8').splitlines()
            reader = DictReader(decoded_file)
            print(reader)
            # Create Hero objects from passed in data
            # ...
            uploadRepresentatives(reader)
            # for representative in reader:
            #     i = 0
            #     print(representative["FULL NAME"]) 
            
            self.message_user(request, "Les élues ont bien été importé")
            print("now update list of Approved emails")
            updateApprovedEmails(self,request)
            return redirect("..")
        form = CsvImportForm()
        payload = {"form": form}
        return render(
            request, "./admin/upload-forms/representative-upload-csv.html", payload
        )

admin.site.register(Representative, RepresentativeAdmin)

#ACTION GROUP PAGE & List
class consultationAdmin(GuardedModelAdmin):
    exclude = ('slug',)
    list_display = ['name','status']
    readonly_fields=('urlid','author','instanceName','instanceHostName')

admin.site.register(ConsultationPublic, consultationAdmin)

#ACTION GROUP PAGE & List
class actionGroupAdmin(GuardedModelAdmin):
    inlines = [ actionStepInline,OrganisationLogoInline,imageToShareInline]
    exclude = ('country','members','actiongroups','conversation','slug',)
    list_display = ['name','status']
    readonly_fields=('urlid','author','instanceName','instanceHostName','representativeEmailSentCounter','twitterClickCounter','peopleTargettedCounter')
    search_fields = ['name'] 
    # fields=('name','test','image') #here to set order of elements in admin
    fieldsets = ( 
        ("Informations de votre instance Risefor (données venant du fichier settings.py)", { 'fields': ('instanceName','instanceHostName')}),
        ("Statistiques de l'action", { 'fields': ('representativeEmailSentCounter','twitterClickCounter','peopleTargettedCounter')}),
        ("Informations générales de votre action", { 'fields': ('urlid','author','name','status','objective','description','actiondate','endactiondate','shortLink','tags')}),
        ("Images bannière",{ 'fields': ('image','mobileImage')}),
        ("Informations supplémentaires", { 'fields': ('titleOrganisationLogoSection','organisation', 'emailOrganisation','linkOrganisation','actioninfolink')}), 
        ("Action Physique : Localisation", { 'fields': ('address', 'city','postcode','lat','lng')}), 
        ("Choix de ciblage", { 'fields': ('typeOfTargetting','targetedOfficials', 'contactPlateforms','addCCToEmail','addPJToEmail')}), 
        ("Contenu pour les Députées", { 'fields': ('electedofficialEmailSubject', 'electedofficialEmailText','TweetMain','Tweet2','Tweet3','Tweet4','representativeCallObjective','representativeCallArgumentation')}), 
        ("Contenu pour les Sénateurices", { 'fields': ('senatorEmailSubject', 'senatorEmailText','senatorTweetMain','senatorTweet2','senatorTweet3','senatorTweet4','senatorCallObjective','senatorCallArgumentation')}),
        ("Contenu pour les Candidates", { 'fields': ('candidateEmailSubject', 'candidateEmailText','candidateTweetMain','candidateTweet2','candidateTweet3','candidateTweet4','candidateCallObjective','candidateCallArgumentation')}),
        ("Ciblage de décisionnnaires précis", { 'fields': ('specificTargetedRepresentatives', 'targetedRepresentativesTitle','targetedEmailSubject','targetedEmailText','targetedTweetMain','targetedTweet2','targetedTweet3','targetedTweet4','targetedCallObjective','targetedCallArgumentation')}),
        ("Communiqué de presse", { 'fields': ('pressreleasetitle', 'pressreleasecontent')}), 
        ("Afficher une vidéo", { 'fields': ('videotitle', 'videourl',)}), 
        ("Afficher une carte des mobilisations", { 'fields': ('mapSignUpSectionTitle', 'mapSignUp',)}), 
        ("Afficher des étapes d'actions", { 'fields': ('actionStepTitle',)}), 
        
    )
    class Media:
        js = ('./js/admin-fields.js',)

admin.site.register(ActionGroup, actionGroupAdmin)



# Emails sent to representatives
class ElectedOfficialContactAdmin(ImportExportModelAdmin,GuardedModelAdmin):
    list_display= ['userName','created_at',]
    list_filter = ['linkedAction',]
admin.site.register(ElectedOfficialContact, ElectedOfficialContactAdmin)


# Emails sent to representatives
class TextSupporterAdmin(ImportExportModelAdmin,GuardedModelAdmin):
    list_display= ['fullname','created_at',]
    list_filter = ['linkedSupport',]
admin.site.register(TextSupporter, TextSupporterAdmin)


# Emails sent to representatives
class NewsletterAdmin(ImportExportModelAdmin,GuardedModelAdmin):
    list_display= ['fullname','created_at',]
    list_filter = []
admin.site.register(Newsletter, NewsletterAdmin)



class contactTeamAdmin(GuardedModelAdmin):
    list_display= ['subject','status','created_at']
admin.site.register(ContactTeam, contactTeamAdmin)


#TRANSLATION FIELDS
class riseforAdmin(TabbedTranslationAdmin):
    group_fieldsets = True
    fieldsets = [
        (u'ActionGroup', {'fields': ('name',)}),
        (u'representativeFunction', {'fields': ('name',)}),
        (u'representativeContactPlateform', {'fields': ('name',)}),
    ]


#Hide flag fields from admin
class CountrieAdmin(admin.ModelAdmin):
    exclude = ('flag','flag_css')
    #pass
admin.site.register(Countrie, CountrieAdmin)


##########
# Upload representatives from csv
##########
class CsvImportForm(forms.Form):
    csv_file = forms.FileField()

# # @admin.register(ActionGroup)
# class uploadRepresentativeAdmin(admin.ModelAdmin):
    
    
