import os
import itertools
import uuid
import unidecode
from datetime import datetime,date
from django.db import models
from django.contrib.auth import get_user_model
from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver
from django.utils.text import slugify
from django.core.mail import EmailMessage
from django.utils.translation import ugettext as _


from django import forms
from djangoldp.models import Model
from djangoldp_conversation.models import Conversation

from djangoldp_i18n.views import I18nLDPViewSet
from django_countries.fields import CountryField,countries
from django_countries.widgets import CountrySelectWidget

import re

from django.conf import settings

from server.settings import *

User = get_user_model()


def cleanAccentuation(text):
    text = unidecode.unidecode(text)
    return text


# appending custom behaviour to User model
def active_actions(self):
    return self.actiongroups.filter(actiondate__gte=date.today())
User.active_actions=active_actions

def convergent_actions(self):
    start_date = datetime(2019, 11, 28)
    end_date = datetime(2019, 12, 9)
    return self.actiongroups.filter(actiondate__gt=start_date,actiondate__lt=end_date)

User.convergent_actions=convergent_actions


class Countrie(Model):
    name = models.CharField(max_length=255, verbose_name=u"Country Code")
    country = CountryField(blank_label='(select country)',countries_flag_url='/static/flags/img/{code}.svg')
    flag = models.CharField(max_length=250,verbose_name=u"Flag", blank=True, null=True)
    flag_css = models.CharField(max_length=250,verbose_name=u"Flag css", blank=True, null=True)

    class Meta: 
        serializer_fields = ['name','country','flag','flag_css']
        nested_fields= ['action_country','representative_country']
    def __str__(self):
        return self.country.code

@receiver(post_save, sender=Countrie)
def save_country_flag(sender, instance,created, **kwargs):
    #print("old names",instance.name,instance.country,instance.country.code)

    if  created:
        if instance.name == 'Global':
            instance.flag = '/static/flags/img/global.svg'
            instance.save()
        else:
            instance.name = instance.country.code
            instance.flag = instance.country.flag
            instance.flag_css = instance.country.flag_css
            #print("new names",instance.name,instance.country,instance.country.code)
            instance.save()


class TagFiltering(Model):
    name = models.CharField(max_length=100, verbose_name=u"Tag name")
    class Meta:
        serializer_fields = ['@id','name']
    def __str__(self):
        return self.name

class ArgumentsToDisplay(Model):
    name = models.CharField(max_length=150, verbose_name=u"titre de l'argument")
    content = models.TextField(verbose_name=u"Description de l'argument")
    image = models.ImageField(max_length=150, verbose_name=u"Image")
    country = models.ManyToManyField(Countrie, related_name="argument_country", verbose_name=u"Langue")
    source = models.URLField(verbose_name=u"source", null=True, blank=True)
    tag = models.ManyToManyField(TagFiltering,verbose_name=u"Type d'argument")

    class Meta:
        serializer_fields = ['@id','name','content','source','tag','image','country']
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add']
        ordering = ['?']

    def __str__(self):
        return self.name


class Tag(Model):
  name = models.CharField(max_length=250, verbose_name=u"Tag name")
  class Meta:
    serializer_fields = ['@id','name']
  def __str__(self):
      return self.name

class visualArguementToDownload(Model):
  name = models.CharField(max_length=150, verbose_name=u"titre de l'argument")
  content = models.TextField(verbose_name=u"Description de l'argument")
  image = models.ImageField(max_length=150, verbose_name=u"Image")
  country = models.ManyToManyField(Countrie, related_name="reasonImage", verbose_name=u"Langue")
  source = models.URLField(verbose_name=u"source", null=True, blank=True)
  tag = models.ManyToManyField(Tag,verbose_name=u"Type d'argument")

  class Meta:
    serializer_fields = ['@id','name','content','source','tag','image','country']
    anonymous_perms = ['view']
    authenticated_perms = ['inherit', 'add']
    ordering = ['?']

  def __str__(self):
    return self.name

class commissionPermanente(Model):
    name = models.CharField(max_length=500, blank=True, null=True, verbose_name=u"Nom de la commission")
    class Meta:
        serializer_fields = ['@id', 'name']
        nested_fields = ['commissionPermanente']
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add', 'change']

    def __str__(self):
        return self.name

class missionParlementaire(Model):
    name = models.CharField(max_length=500, blank=True, null=True, verbose_name=u"Nom de la commission")
    class Meta:
        serializer_fields = ['@id', 'name']
        nested_fields = ['missionParlementaire']
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add', 'change']
    
    def __str__(self):
        return self.name



class additionalFunction(Model):
    name = models.CharField(max_length=500, blank=True, null=True, verbose_name=u"Nom de la commission")
    class Meta:
        serializer_fields = ['@id', 'name']
        nested_fields = []
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add', 'change']

    def __str__(self):
        return self.name


class groupeEtude(Model):
    name = models.CharField(max_length=500, blank=True, null=True, verbose_name=u"Nom de la commission")
    
    class Meta:
        serializer_fields = ['@id', 'name']
        nested_fields = []
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add', 'change']

    def __str__(self):
        return self.name


class parliamentaryPosition(Model):
    name = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Nom de la commission")
    class Meta:
        serializer_fields = ['@id', 'name']
        nested_fields = []
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add', 'change']

    def __str__(self):
        return self.name

class representativeFunction(Model):
    name = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Fonction de l'élu")

    class Meta:
        nested_fields = []
        authenticated_perms = ['inherit', 'add', 'change']

    def __str__(self):
        return self.name

class Representative(Model):
    """Displaying contact and functions of elected officials"""
    #global information
    civility = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Civilité")
    firstname = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Prénom")
    lastname = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Nom")
    fullname = models.CharField(max_length=500, blank=True, null=True, verbose_name=u"Nom complet")
    function = models.ForeignKey(representativeFunction,blank=True,null=True, related_name="representativePosition", verbose_name=u"Fonction",
                                       on_delete=models.CASCADE)
    title = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Titre")
    politicalParty = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Parti politique")
    photo = models.ImageField(verbose_name=u"Photo")
    slug = models.SlugField(blank=True, null=True, unique=True)
    
    #Location info
    election_department = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Département d'élection")
    election_region = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Région d'élection")
    address1 = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Adresse 1")
    address2 = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Adresse 2")
    postal_code = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Code postal")
    city = models.CharField(max_length=150, blank=True, null=True, verbose_name=u"Ville")
    allPostalCodes = models.TextField(blank=True, verbose_name=u"Tous les codes postaux de la circonscription")
    #link to country
    representativeCountry = models.ForeignKey(Countrie, related_name='representative_country', blank=True, null=True, verbose_name=u"Pays",on_delete=models.SET_NULL)
    
    #Activity information
    permanentCommission = models.ForeignKey(commissionPermanente,blank=True,null=True, related_name="commissionPermanente", verbose_name=u"Commission permanente",
                                       on_delete=models.CASCADE)
    permanentCommissionFunction = models.ForeignKey(parliamentaryPosition,blank=True,null=True, related_name="commissionPermanenteFunction", verbose_name=u"Fonction commission permanente",
                                       on_delete=models.CASCADE)

    parliamentaryMissions = models.ManyToManyField(missionParlementaire,blank=True, related_name="missionParlementaire", verbose_name=u"missions parlementaires")
    
    additionalFunctions = models.ManyToManyField(additionalFunction,blank=True, related_name="additionalFunction", verbose_name=u"Fonctions additionnels") 
    
    studyGroups = models.ManyToManyField(groupeEtude,blank=True, related_name="study_groups", verbose_name=u"Groupes d'études")
    
    #Contact information
    assistants = models.ManyToManyField('Representative', blank=True)

    #position on campaign -> needs to be refactored to allow this info
    #to be changed depending on the action
    position_campagne = models.CharField(max_length=255, blank=True, verbose_name=u"Position de l'élue sur la campange")

        
    #new fields
    circonscription = models.CharField(max_length=50, blank=True, null=True, verbose_name=u"Circonscription")
    profession = models.CharField(max_length=150, blank=True, null=True, verbose_name=u"Metier")
    facebook = models.CharField(max_length=150, blank=True, null=True, verbose_name=u"Facebook")
    website = models.CharField(max_length=300, blank=True, null=True, verbose_name=u"Site web")

    class Meta:
        serializer_fields = ['@id', 'civility','facebook','profession','circonscription', 'firstname', 'lastname', 'fullname', 'function', 'title', 'politicalParty', 'election_department', 'election_region', 'directPhones','directEmails','twitterAccount', 'address1', 'address2', 'postal_code', 'city', 'representativeCountry', 'website', 'photo', 'slug', 'permanentCommission','permanentCommissionFunction', 'additionalFunctions', 'parliamentaryMissions', 'studyGroups', 'assistants','allPostalCodes','position_campagne']
        nested_fields = ["directPhones","directEmails","twitterAccount"]
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add', 'change']
        ordering = ['fullname']
    """
    def fullname(self):
        self.fullname = "{} {}".format(self.firstname, self.lastname)
        return "{} {}".format(self.firstname, self.lastname)
    """
    def __str__(self):
        if self.firstname and self.lastname:
            return self.firstname + " " + self.lastname
        elif self.fullname:
            return self.fullname
        else:
            return "no name"

###
# Model linked to email security breach 
# Objective : Restrict emails being sent to representatives in the database
# More info : https://git.risefor.org/applications/risefor-front/-/issues/49
###
class ApprovedEmail(Model):
    created_at = models.DateTimeField(auto_now_add=True)
    name = models.CharField(max_length=150, blank=True, null=True, verbose_name=u"Email")
    class Meta:
        serializer_fields =['@id','created_at','name']
        nested_fields=[]
        anonymous_perms=[]
        authenticated_perms = ['inherit', 'add']
        owner_perms = ['inherit', 'change', 'control', 'delete']
        ordering = ['-created_at']

    def __str__(self):
        return self.name 


class Phone(Model):
    """Phone numbers of elected officials"""
    numberLocation = models.CharField(max_length=50,default='accueil',null=True, verbose_name=u"Nom du lieu de ce numéro")
    number = models.CharField(max_length=20, verbose_name=u"Numéro de téléphone")
    representative = models.ForeignKey(Representative,related_name="directPhones", verbose_name=u"Élu",
                                       on_delete=models.CASCADE)
    def __str__(self):
        if self.numberLocation:
            return self.representative.fullname + " " + self.numberLocation
        else:
            return self.representative.fullname

class Email(Model):
    """Emails of elected officials"""
    name = models.CharField(max_length=150, blank=True, null=True, verbose_name=u"Email")
    representative = models.ForeignKey(Representative,blank=True, null=True, related_name="directEmails", verbose_name=u"Elu",
                                       on_delete=models.CASCADE)
    class Meta(Model.Meta):
        container_path = '/api/directEmails/'

    def __str__(self):
        return self.name

class TwitterAccount(Model):
    """Emails of elected officials"""
    name = models.CharField(max_length=150, blank=True, null=True, verbose_name=u"Compte twitter")
    representative = models.ForeignKey(Representative,blank=True, null=True, related_name="twitterAccount", verbose_name=u"Elu",
                                       on_delete=models.CASCADE)

    def __str__(self):
        return self.name


class Theme(Model):
    """Type of ActionGroup"""
    name = models.CharField(max_length=255, verbose_name=u"Nom")
    photo = models.URLField(max_length=9999999, blank=True, null=True, verbose_name=u"Photo")

    def __str__(self):
        return self.name


class OrgType(Model):
    """Type of organisation. E.g. NGO, Citizen Movement.."""
    name = models.CharField(max_length=255, verbose_name=u"Type d'organisation")

    def __str__(self):
        return self.name
  

class representativeContactPlateform(Model):
    name = models.CharField(max_length=255, verbose_name=u"Plateforme d'interpellation")

    def __str__(self):
        return self.name


class actionMapSignUp(Model):
    created_at = models.DateTimeField(auto_now_add=True)
    name = models.CharField(max_length=255,verbose_name=u"Nom de l'action")
    status = models.CharField(max_length=32, default='to-come', choices=(('today', 'Aujourd\'hui'),('to-come', 'A venir'),('passed', 'Passé')), verbose_name="Status de l'événement")
    road = models.CharField(max_length=255,blank=True,null=True,verbose_name=u"Rue du rendez vous")
    city = models.CharField(max_length=255,verbose_name=u"Ville du rendez vous")
    link = models.CharField(max_length=255,blank=True, null=True,verbose_name=u"Lien")
    organisedBy = models.CharField(max_length=255,blank=True,null=True,verbose_name=u"Organisé par")
    contactEmail = models.CharField(max_length=255,blank=True,null=True,verbose_name=u"Contact Email")
    dateDePassage = models.DateField(max_length=255,default='',verbose_name=u"Date")
    time = models.TimeField(blank=True, null=True,verbose_name=u"Heure du rdv")
    lat = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=u"Latitude")
    lng = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=u"Longitude")

    class Meta:
        serializer_fields =['@id','status','created_at','name','link','dateDePassage','time','road','city','contactEmail','organisedBy',
        'lat','lng']
        nested_fields=['actionSignUps']
        anonymous_perms=['view','add']
        authenticated_perms = ['inherit', 'add']
        owner_perms = ['inherit', 'change', 'control', 'delete']
        ordering = ['created_at']

    def __str__(self):
        return self.city + " " + self.name
"""
Use new event to update status of all event, based on their date
    # Get date of event, 
        If passed, set it as passed
    ### Evolution : 
        > Use Celery in order to run once a day
        > Could also be used on action status's when a date in informed
"""
@receiver(post_save, sender=actionMapSignUp)
def save_setstatus_actionmapsignup(sender, instance, created, **kwargs):
    if created:
        allMapActions =  actionMapSignUp.objects.all()
        today = datetime.today().strftime('%Y-%m-%d')
        for action in allMapActions:
            eventDate = action.dateDePassage
            if eventDate < date.today():
                action.status = 'passed'
            elif eventDate == date.today():
                action.status = 'today'
            else:
                action.status = 'to-come'
            action.save()


class financeCampaign(Model):
    created_at = models.DateTimeField(auto_now_add=True)
    name = models.CharField(max_length=255, verbose_name=u"Nom")
    status = models.CharField(max_length=32, default='active', choices=(('active', 'Active'),('pending', 'Pending'), ('funded', 'Funded'), ('abandonned', 'Abandonned')), verbose_name="Finance campaign status")
    image = models.ImageField(verbose_name=u"Image de la campagne", null=True, blank=True)
    link = models.URLField(max_length=9999999, blank=True, null=True,verbose_name=u"Lien vers la campagne")
    
    class Meta:
        serializer_fields =['@id','created_at','name','status','image','link']
        nested_fields=[]
        anonymous_perms=['view']
        authenticated_perms = ['inherit', 'add']
        owner_perms = ['inherit', 'change', 'control', 'delete']
        ordering = ['created_at']

    def __str__(self):
        return self.name

class UrlShortner(Model):
    actionName = models.CharField(max_length=200,verbose_name=u"Nom de la ressource ciblée (action ou consultation), généré automatiquement")
    url = models.CharField(max_length=200,verbose_name=u"Urlid de l'action ciblé, généré automatiquement")
    shortLink = models.CharField(max_length=200,verbose_name=u"Lien court, généré automatiquement et personnalisable")
    class Meta:
        nested_fields = ['actionShortLink','consultationShortLink','appelShortLink']

    def __str__(self):
        print(f"Short Url for: {self.url} is {self.shortLink}")
        if self.actionName :
            return self.actionName 
        else:
            return self.url

class ConsultationPublic(Model):
    """Connected users can create actions that are visible to all users in the group"""
    created_at = models.DateTimeField(auto_now_add=True)
    author = models.ForeignKey(User, related_name='created_consultation', blank=True, null=True, on_delete=models.SET_NULL)
    name = models.CharField(max_length=255, verbose_name=u"Nom de la consultation")
    status = models.CharField(max_length=32, default='active', choices=(('active', 'Active'),('pending', 'Pending'), ('inappropriate', 'Inappropriate'), ('finished', 'Finished Action')), verbose_name="action status")
    instanceName = models.CharField(max_length=255, verbose_name=u"Nom de l'instance",default=settings.INSTANCE_NAME)
    instanceHostName = models.CharField(max_length=255, verbose_name=u"Domaine de l'instance",default=settings.INSTANCE_DOMAINE_NAME)
    objective = models.TextField(verbose_name=u"Objectif")
    description = models.TextField(blank=True, null=True, verbose_name=u"Contexte / Description")
    enddate = models.DateField(blank=True, null=True, verbose_name=u"Date de fin de la consultation")
    consultationLink = models.URLField(max_length=9999999, blank=True, null=True,verbose_name=u"Lien vers la consultation")
    
    image = models.ImageField(verbose_name=u"Image Tablette et Ordinateur", null=True, blank=True)
    mobileImage = models.ImageField(verbose_name=u"Image Mobile", null=True, blank=True)
    shortLink = models.ForeignKey(UrlShortner, verbose_name=u"Short Link",related_name='consultationShortLink', null=True,blank=True,
                                          on_delete=models.SET_NULL)
    
    # more info on action
    infolink = models.URLField(max_length=9999999, blank=True, null=True,verbose_name=u"Lien 'En savoir plus'")
    
    argumentaire = models.TextField(blank=True, null=True,
                                        verbose_name=u"Argumentaire pour la consultation - Version 1")
    argumentaire2 = models.TextField(blank=True, null=True,
                                        verbose_name=u"Argumentaire pour la consultation - Version 2")
    argumentaire3 = models.TextField(blank=True, null=True,
                                        verbose_name=u"Argumentaire pour la consultation - Version 3")
    argumentaire4 = models.TextField(blank=True, null=True,
                                        verbose_name=u"Argumentaire pour la consultation - Version 4")
    argumentaire5 = models.TextField(blank=True, null=True,
                                        verbose_name=u"Argumentaire pour la consultation - Version 5")
    argumentaire6 = models.TextField(blank=True, null=True,
                                        verbose_name=u"Argumentaire pour la consultation - Version 6")
    argumentaire7 = models.TextField(blank=True, null=True,
                                        verbose_name=u"Argumentaire pour la consultation - Version 7")
    argumentaire8 = models.TextField(blank=True, null=True,
                                        verbose_name=u"Argumentaire pour la consultation - Version 8")
    argumentaire9 = models.TextField(blank=True, null=True,
                                        verbose_name=u"Argumentaire pour la consultation - Version 9")
    argumentaire10 = models.TextField(blank=True, null=True,
                                        verbose_name=u"Argumentaire pour la consultation - Version 10")
    

    #elements below not in admin fieldset
    slug = models.SlugField(blank=True, null=True, unique=True)
    
    address = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Adresse")
    city = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Ville")
    postcode = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Code postal")
    lat = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=u"Latitude")
    lng = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=u"Longitude")
    tags = models.ManyToManyField(Tag,verbose_name=u"Consultation tags",blank=True,null=True)


    class Meta:
        auto_author = 'author'
        serializer_fields = ['@id','created_at','name','status','instanceName','instanceHostName','objective','description','enddate','image','mobileImage','shortLink','infolink','slug','address','city','postcode','lat','lng','tags','consultationLink','argumentaire','argumentaire2','argumentaire3','argumentaire4','argumentaire5','argumentaire6','argumentaire7','argumentaire8','argumentaire9','argumentaire10']
        nested_fields = []
        anonymous_perms = ['view','add']
        authenticated_perms = ['inherit', 'add']
        owner_perms = ['inherit', 'change', 'control', 'delete']
        owner_field = 'author'
        ordering = ['-created_at']
        view_set = I18nLDPViewSet
        rdf_type = 'risefor:consultationpublic'
        # container_path = 'consultationpublic/'
        # lookup_field = 'id'
    def __str__(self):
        return self.name



        
class ActionGroup(Model):
    """Connected users can create actions that are visible to all users in the group"""
    created_at = models.DateTimeField(auto_now_add=True)
    name = models.CharField(max_length=255, verbose_name=u"Nom de l'action")
    status = models.CharField(max_length=32, default='active', choices=(('active', 'Active'),('pending', 'Pending'), ('inappropriate', 'Inappropriate'), ('finished', 'Finished Action')), verbose_name="action status")
    instanceName = models.CharField(max_length=255, verbose_name=u"Nom de l'instance",default=settings.INSTANCE_NAME)
    instanceHostName = models.CharField(max_length=255, verbose_name=u"Domaine de l'instance",default=settings.INSTANCE_DOMAINE_NAME)
    objective = models.TextField(verbose_name=u"Objectif")
    description = models.TextField(blank=True, null=True, verbose_name=u"Contexte / Description")
    
    actiondate = models.DateField(blank=True, null=True, verbose_name=u"Date de l'action")
    endactiondate = models.DateField(blank=True, null=True, verbose_name=u"Date de fin de l'action")
    image = models.ImageField(verbose_name=u"Image Tablette et Ordinateur", null=True, blank=True)
    mobileImage = models.ImageField(verbose_name=u"Image Mobile", null=True, blank=True)
    shortLink = models.ForeignKey(UrlShortner, verbose_name=u"Short Link",related_name='actionShortLink', null=True,blank=True,
                                          on_delete=models.SET_NULL)
    
    titleOrganisationLogoSection = models.CharField(max_length=255,blank=True, null=True, verbose_name=u"Titre au dessus de la section des logos")
    organisation = models.CharField(max_length=255,blank=True, null=True, verbose_name=u"Organisation associé à l'action")
    emailOrganisation = models.CharField(max_length=500, blank=True, null=True,verbose_name=u"Email de contact de l'organisation")
    linkOrganisation = models.URLField(max_length=500, blank=True, null=True,verbose_name=u"Lien 'en savoir plus sur le collectif'")
    # more info on action
    actioninfolink = models.URLField(max_length=500, blank=True, null=True,verbose_name=u"Lien 'En savoir plus'")
    
    #5 elements below not in admin fieldset
    members = models.ManyToManyField(User, related_name='actiongroups', blank=True)
    actiongroups = models.ManyToManyField("self", blank=True)
    author = models.ForeignKey(User, related_name='created_groups', blank=True, null=True, on_delete=models.SET_NULL)
    conversation = models.ManyToManyField(Conversation, blank=True)
    slug = models.SlugField(blank=True, null=True, unique=True)
    
    
    address = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Adresse")
    city = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Ville")
    postcode = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Code postal")
    lat = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=u"Latitude")
    lng = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=u"Longitude")
    tags = models.ManyToManyField(Tag,verbose_name=u"Action tags",blank=True,null=True)


    #link to country
    country = models.ForeignKey(Countrie, related_name='action_country', blank=True, null=True, on_delete=models.SET_NULL)
    
    #count emails
    representativeEmailSentCounter = models.IntegerField(default="0",verbose_name=u"Nombre d'email envoyé",blank=True,null=True)
    twitterClickCounter = models.IntegerField(default="0",verbose_name=u"Nombre de tweet envoyé",blank=True,null=True)
    peopleTargettedCounter = models.CharField(max_length=200,verbose_name=u"Nombre et type de personnes ciblés",blank=True,null=True)

    #Representative Targeting choices
    typeOfTargetting = models.CharField(max_length=32,blank=True, default='', choices=(('postalCode', 'Par code postal'),('byGeography', 'Par département'), ('specificRepresentatives', 'Je choisi certaines personnes'),('international', 'Par pays'),('none', "Pas d'interpellation")), verbose_name="Type de ciblage")
    targetedOfficials = models.ManyToManyField(representativeFunction,blank=True, related_name="targetedRepresentatives", verbose_name=u"Type d'élues ciblés. Si vous ciblez des décisionnaires précis, laissez ce champs vide")
    contactPlateforms = models.ManyToManyField(representativeContactPlateform,blank=True, related_name="contactPlateforms", verbose_name=u"Plateforme d'interpellations")

    addCCToEmail = models.CharField(max_length=500,blank=True, null=True, verbose_name=u"Ajouter des CC à tous les emails envoyé. Pour plusieurs CC, veuillez les séparer avec une ','")
    addPJToEmail = models.FileField(blank=True, null=True, verbose_name=u"Ajouter une PJ à l'email envoyé")

    pressreleasetitle = models.CharField(max_length=255,blank=True, null=True, verbose_name=u"Communiqué de Presse - Titre")
    pressreleasecontent = models.FileField(blank=True, null=True, verbose_name=u"Communiqué de Presse - PDF")

    # elected officials email
    electedofficialEmailSubject = models.CharField(max_length=255,blank=True, null=True,
                                                   verbose_name=u"Sujet de l'email à  envoyer aux députées")
    electedofficialEmailText = models.TextField(blank=True, null=True,
                                                verbose_name=u"Texte de l'email à envoyer aux députées")
    # senator email
    senatorEmailSubject = models.CharField(max_length=255,blank=True, null=True,
                                                   verbose_name=u"Sujet de l'email à envoyer aux sénateurices")
    senatorEmailText = models.TextField(blank=True, null=True,
                                                verbose_name=u"Texte de l'email à envoyer aux sénateurices")
    # candidate email
    candidateEmailSubject = models.CharField(max_length=255,blank=True, null=True,
                                                   verbose_name=u"Sujet de l'email à envoyer aux candidat.e.s")
    candidateEmailText = models.TextField(blank=True, null=True,
                                                verbose_name=u"Texte de l'email à envoyer aux candidat.e.s")    

    #specific targeted people
    
    specificTargetedRepresentatives = models.ManyToManyField(Representative,verbose_name=u"Cible précisement certains élu-e-s",blank=True,null=True)
    # -> title of section here to allow dynamic
    # mused be informed to display section
    targetedRepresentativesTitle = models.CharField(max_length=255, verbose_name=u"Cible précise - Titre de la section",blank=True, null=True,)
    targetedEmailSubject = models.CharField(max_length=255,blank=True, null=True,
                                                   verbose_name=u"Cible précise - Sujet de l'email")
    targetedEmailText = models.TextField(blank=True, null=True,
                                                verbose_name=u"Cible précise - Texte de l'email")    

    
    #interpellation députée on twitter
    TweetMain = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Députée - Main Tweet")
    Tweet2 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Députée - Tweet 2")
    Tweet3 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Députée - Tweet 3")
    Tweet4= models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Députée - Tweet 4")
    
    #interpellation senators on twitter
    senatorTweetMain = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"senator - Main Tweet")
    senatorTweet2 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"senator - Tweet 2")
    senatorTweet3 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"senator - Tweet 3")
    senatorTweet4= models.CharField(blank=True, null=True,max_length=350, verbose_name=u"senator - Tweet 4")
    
    #interpellation candidates on twitter
    candidateTweetMain = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"candidate - Main Tweet")
    candidateTweet2 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"candidate - Tweet 2")
    candidateTweet3 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"candidate - Tweet 3")
    candidateTweet4= models.CharField(blank=True, null=True,max_length=350, verbose_name=u"candidate - Tweet 4")
    
    #interpellation targeted on twitter
    targetedTweetMain = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Cible précise - Main Tweet")
    targetedTweet2 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Cible précise - Tweet 2")
    targetedTweet3 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Cible précise - Tweet 3")
    targetedTweet4= models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Cible précise - Tweet 4")
    
    

    #interpellation par téléphone
    representativeCallObjective = models.CharField(blank=True, null=True, max_length=280, verbose_name=u"Députée - Objectif de l'appel")
    representativeCallArgumentation = models.TextField(blank=True, null=True, verbose_name=u"Députée - Argumentaire pour l'appel")

    senatorCallObjective = models.CharField(blank=True, null=True, max_length=280, verbose_name=u"Sénateurice - Objectif de l'appel")
    senatorCallArgumentation = models.TextField(blank=True, null=True, verbose_name=u"Sénateurice - Argumentaire pour l'appel")

    candidateCallObjective = models.CharField(blank=True, null=True, max_length=280, verbose_name=u"Candidat - Objectif de l'appel")
    candidateCallArgumentation = models.TextField(blank=True, null=True, verbose_name=u"Candidat - Argumentaire pour l'appel")

    targetedCallObjective = models.CharField(blank=True, null=True, max_length=280, verbose_name=u"Specific target - Objectif de l'appel")
    targetedCallArgumentation = models.TextField(blank=True, null=True, verbose_name=u"Specific target - Argumentaire pour l'appel")

    
    #call video
    videotitle = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Titre de la vidéo")
    videourl = models.URLField(max_length=9999999, blank=True, null=True, verbose_name=u"Vidéo associé à l'action")

    #Signups 
    mapSignUpSectionTitle = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Titre de la section Carte")
    mapSignUp = models.ManyToManyField(actionMapSignUp, blank=True, related_name="actionSignUps")
    
    #Action Step title (each step in nested fields)
    actionStepTitle = models.TextField(verbose_name=u"Titre de la section 'étape' de l'action",blank=True,null=True)

    
    class Meta:
        auto_author = 'author'
        serializer_fields = ['@id','shortLink', 'name', 'author', 'pressreleasetitle', 'pressreleasecontent','status',
                             'objective', 'actiondate','organisation','emailOrganisation','linkOrganisation','actioninfolink', 
                             'endactiondate', 'image','mobileImage', 'description','country',
                             'electedofficialEmailSubject', 'electedofficialEmailText','targetedEmailSubject','targetedEmailText',
                             'members','instanceName','instanceHostName',
                             'targetedCallObjective','targetedCallArgumentation','representativeCallObjective','representativeCallArgumentation','senatorCallObjective','senatorCallArgumentation',
                             'actiongroups', 'conversation', 'address', 'city', 'postcode', 'lat', 'lng',
                             'events','videotitle',
                             'videourl','TweetMain','Tweet2','Tweet3','Tweet4',
                             'senatorEmailSubject','senatorEmailText',
                             'senatorTweetMain','senatorTweet2','senatorTweet3','senatorTweet4','targetedTweetMain','targetedTweet2','targetedTweet3','targetedTweet4',
                             'candidateEmailSubject','candidateEmailText',
                             'candidateTweetMain','candidateTweet2','candidateTweet3','candidateTweet4',
                             'candidateCallObjective','candidateCallArgumentation',
                             'typeOfTargetting','targetedOfficials','contactPlateforms','addCCToEmail','addPJToEmail',
                             'mapSignUpSectionTitle','mapSignUp','actionStep','representativeEmailSentCounter','twitterClickCounter','peopleTargettedCounter',
                             'specificTargetedRepresentatives','targetedRepresentativesTitle',
                             'tags',"supportOrganisationLogo","titleOrganisationLogoSection",
                             'shareImages']
        nested_fields = ["author", "actiongroups", "conversation", "members", "events",
                         'actionEmailSentRelated',
                         "actionStep","supportOrganisationLogo",
                         'shareImages'
                         ]
        anonymous_perms = ['view','add']
        authenticated_perms = ['inherit', 'add']
        owner_perms = ['inherit', 'change', 'control', 'delete']
        owner_field = 'author'
        ordering = ['-created_at']
        view_set = I18nLDPViewSet
        rdf_type = 'risefor:actiongroup'
        container_path = 'actiongroups/'
        # lookup_field = 'id'
    def __str__(self):
        return self.name

class OrganisationLogoForAction(Model):
    created_at = models.DateTimeField(auto_now_add=True)
    orgName = models.CharField(max_length=255, verbose_name=u"Nom de l'organisation") 
    image = models.ImageField(verbose_name=u"Logo", null=False, blank=False)
    linkedAction = models.ForeignKey(ActionGroup, related_name='supportOrganisationLogo',null=True, blank=False, on_delete=models.SET_NULL)

    class Meta:
        serializer_fields =['@id','created_at','orgName','image','linkedAction']
        # nested_fields=['']
        anonymous_perms=['view']
        authenticated_perms = ['inherit', 'add']
        owner_perms = ['inherit', 'change', 'control', 'delete']
        ordering = ['created_at']

    def __str__(self):
        return self.orgName 


class imageToShare(Model):
  image = models.FileField(verbose_name=u"Image à partager")
  linkedAction = models.ForeignKey(ActionGroup, related_name='shareImages',null=True, blank=False, on_delete=models.SET_NULL)

  class Meta:
    serializer_fields = ['@id','image','linkedAction']
    # nested_fields = ['']
    anonymous_perms = ['view']
    authenticated_perms = ['inherit', 'add']

  def __str__(self):
      name = str(self.image)
      return name


class ActionStep(Model):
    order = models.IntegerField(verbose_name=u"Ordre (1,2,3)")
    name = models.CharField(max_length=50, blank=True, null=True, verbose_name=u"Titre de l'étape")
    description =  models.TextField(blank=True, null=True, verbose_name=u"Description rapide")
    actiongroup = models.ForeignKey(ActionGroup,blank=True, null=True, related_name="actionStep", verbose_name=u"Action associé", on_delete=models.SET_NULL)
    active = models.BooleanField(default=False,verbose_name=u"Etape en cours")

    def __str__(self):
        return self.name

class ElectedOfficialContact(Model):
    author = models.ForeignKey(User, blank=True, null=True, on_delete=models.SET_NULL)
    created_at = models.DateTimeField(auto_now_add=True)
    subject = models.CharField(max_length=255,verbose_name=u"Sujet de l'email envoyé aux élu.e.s")
    content = models.TextField(verbose_name=u"Contenu de l'email envoyé aux élu.e.s")
    userName = models.TextField(verbose_name=u"Name of citizen")
    userEmail = models.CharField(max_length=255, verbose_name=u"User email")
    representativeEmails = models.CharField(max_length=1000, verbose_name=u"Adresses ciblés")
    optin1 =  models.BooleanField(default=True,verbose_name=u"Accept saving email for sending")
    optin2 =  models.BooleanField(default=False,blank=True, null=True,verbose_name=u"Sign up for Risefor Notifications")
    linkedAction = models.ForeignKey(ActionGroup,blank=True,null=True,related_name="actionEmailSentRelated",verbose_name=u"Action liée",on_delete=models.SET_NULL)
    
    class Meta:
        auto_author = 'author'
        serializer_fields =['@id','author','subject','content','userName','userEmail','representativeEmails','optin1','optin2','linkedAction']
        nested_fields=['linkedAction']
        anonymous_perms=['add']
        authenticated_perms = ['inherit', 'add']
        owner_perms = ['inherit', 'change', 'control', 'delete']
        owner_field = 'author'
        ordering = ['created_at']

        view_set = I18nLDPViewSet
        rdf_type = 'risefor:contactRepresentatives'
        container_path = 'electedofficialcontacts/'
    
    def __str__(self):
        return self.userName


class AppelSoutien(Model):
    created_at = models.DateTimeField(auto_now_add=True)
    name = models.CharField(max_length=255, verbose_name=u"Nom de l'appel")
    status = models.CharField(max_length=32, default='active', choices=(('active', 'Active'),('pending', 'Pending'), ('inappropriate', 'Inappropriate'), ('finished', 'Finished Action')), verbose_name="action status")
    objective = models.TextField(verbose_name=u"Objectif")
    description = models.TextField(blank=True, null=True, verbose_name=u"Contexte / Description (préambule)")
    image = models.ImageField(verbose_name=u"Image bannière", null=True, blank=True)
    mobileImage = models.ImageField(verbose_name=u"Image Mobile", null=True, blank=True)
    
    documentTelecharger = models.FileField(blank=True, null=True, verbose_name=u"Document à télécharger - PDF")

    emailConfirmationSujet =  models.CharField(max_length=250, default='adresse',verbose_name=u"Email de confirmation - Sujet")
    emailConfirmationContenu = models.TextField(verbose_name=u"Contenu du mail - Vous pouvez ajouter des balises html tel que <a href>")
    emailConfirmationReplyTo = models.EmailField(max_length=200,default='noreply@risefor.org', verbose_name=u"Email de réponse pour les mails de confirmation")

    shortLink = models.ForeignKey(UrlShortner, verbose_name=u"Short Link",related_name='appelShortLink', null=True,blank=True,
                                          on_delete=models.SET_NULL)
    

    class Meta:
        serializer_fields =['@id','created_at','name','mobileImage','status','objective','description','image','documentTelecharger','emailConfirmationSujet','emailConfirmationContenu','emailConfirmationReplyTo','shortLink']
        nested_fields=[]
        anonymous_perms=['view']
        authenticated_perms = ['inherit', 'add']
        owner_perms = ['inherit', 'change', 'control', 'delete']
        ordering = ['created_at']

    def __str__(self):
        return self.name 

###
# Short link creater
###
@receiver(post_save, sender=AppelSoutien)
@receiver(post_save, sender=ConsultationPublic)
@receiver(post_save, sender=ActionGroup)
def save_createShortLink_actiongroup(sender, instance, created, **kwargs):
    import random
    try: 
        existingUrls = UrlShortner.objects.get(url=instance.urlid)    
    except UrlShortner.DoesNotExist:
        actionName = instance.name.replace(" ","-")
        actionName = actionName.replace(",","")
        actionName = actionName.replace("'","")
        actionName = actionName.replace("?","")
        actionName = actionName.replace("|","")
        actionName = actionName.replace("(","")
        actionName = actionName.replace(")","")
        actionName = actionName.replace(":","")
        actionName = actionName.replace("'","")
        actionName = actionName.lower()
        actionName = cleanAccentuation(actionName)
        # print("new action name",actionName)
        actionSlug = settings.INSTANCE_DOMAINE_NAME + "/s/" + actionName + "-" + str(random.randrange(1,10000))
        
        senderName = sender.__name__
        print('sender is',senderName)
        if 'Action' in senderName:
            categoryText = "Lien pour l'action : "
        elif 'Consultation' in senderName:
            categoryText="Lien pour la consultation : "
        elif 'Appel' in senderName:
            categoryText="Lien pour l'appel : "

        shortUrl = UrlShortner(
            actionName=categoryText+instance.name,
            url=instance.urlid,shortLink = actionSlug)
        shortUrl.save()


#short Link created, get it & update group value
@receiver(post_save, sender=AppelSoutien)
@receiver(post_save, sender=ConsultationPublic)
@receiver(post_save, sender=ActionGroup)
def save_updateGroupShortLink_actiongroup(sender, instance, created, **kwargs):
    # print('shortlink field',instance.shortLink)
    if instance.shortLink == None:
        existingShortLinks = UrlShortner.objects.all()
        actionGroupLink = existingShortLinks.get(url=instance.urlid)
        instance.shortLink = actionGroupLink
        instance.save(update_fields=['shortLink'])


#Mobile image handling - If no image provided, take the desktop image and save it as mobile version
@receiver(post_save, sender=ActionGroup)
@receiver(post_save, sender=ConsultationPublic)
@receiver(post_save, sender=AppelSoutien)
def save_emptyMobileImage_actiongroup(sender, instance, created, **kwargs):
    if (instance.mobileImage == None) or (instance.mobileImage == '') :
        #if no desktop image, just save 
        if (instance.image == None) or (instance.image == '') :
            instance.save
        #else, get the desktop image & save it as mobile
        else:
            desktopImage = instance.image
            print("empty mobile image",desktopImage) 
            instance.mobileImage = desktopImage
            print("mobile image updated",instance.mobileImage) 
            instance.save(update_fields=['mobileImage'])


@receiver(post_save, sender=ActionGroup)
def save_updateTargetCounter_actiongroup(sender, instance, created, **kwargs):
    #get the type of targeting used
    targettingStyle = instance.typeOfTargetting
    actionTargettingTotal = ""
    ### ciblage par choix des cibles
    if targettingStyle == 'specificRepresentatives':
        # Update the field value directly without saving the instance
        ActionGroup.objects.filter(pk=instance.pk).update(
            peopleTargettedCounter=str(instance.specificTargetedRepresentatives.count()) + " décisionnaires ciblés"
        )
    ### ciblage par localité
    elif targettingStyle == 'postalCode':
        contentToAdd = []
        for official in instance.targetedOfficials.all():
            print("getting total of reps")
            print(official.name,official.id)
            print(Representative.objects.filter(function__id=official.id).count())
            contentToAdd.append(str(Representative.objects.filter(function__id=official.id).count())+" "+str(official.name))

        displayContent = ", ".join(contentToAdd)
        ActionGroup.objects.filter(pk=instance.pk).update(
            peopleTargettedCounter=displayContent+" ciblés"
        )


class Event(Model):
    """An event, as associated with an Action"""
    eventName = models.CharField(max_length=255, verbose_name=u"Nom de l'évenement")
    image = models.URLField(verbose_name=u"Image", null=True, blank=True)
    date = models.DateField(blank=True, null=True, verbose_name=u"Date de l'évenement")
    location = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Lieu")
    theme = models.ForeignKey(Theme, verbose_name=u"Type d'événement", null=True, on_delete=models.SET_NULL)
    organization = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Organisation")
    actiongroup = models.ForeignKey(ActionGroup, related_name="events", blank=True, null=True, on_delete=models.SET_NULL)
    eventDescription = models.TextField(blank=True, null=True, verbose_name=u"Description")

    class Meta:
        serializer_fields = ['@id','image', 'eventName', 'eventDescription', 'date', 'location', 'theme', 'actiongroup', 'organization']
        ordering=['-date']

    def __str__(self):
        return self.eventName

# HelpInstall - one kind of contact form (submitting a request for help installing the tool)


class HelpOrgType(Model):
    """Type of organisation wanting to install the tool"""
    name = models.CharField(max_length=255, verbose_name=u"Type de collectif")

    def __str__(self):
        return self.name


class HelpLevel(Model):
    """A category representing the amount of help needed"""
    name = models.CharField(max_length=255, verbose_name=u"Niveau d'aide nécessaire")

    def __str__(self):
        return self.name


class HelpInstall(Model):
    """Contact form to ask for help in order to deploy Risefor"""
    created_at = models.DateTimeField(auto_now_add=True)
    orgname = models.CharField(max_length=255, verbose_name=u"Nom de l'organisation")
    orgType = models.ForeignKey(HelpOrgType, verbose_name=u"Type de collectif", null=True, on_delete=models.SET_NULL)
    helpLevel = models.ForeignKey(HelpLevel, verbose_name=u"Level of technical help needed", null=True, on_delete=models.SET_NULL)
    usage_tool = models.TextField(verbose_name=u"Cadre d'utilisation")
    extra_msg = models.TextField(verbose_name=u"Message supplémentaire")
    contactEmail = models.EmailField(max_length=400, verbose_name=u"Adresse email du contact")

    class Meta:
        serializer_fields = ['@id', 'created_at', 'extra_msg', 'usage_tool', 'orgname', 'orgType', 'helpLevel',
                             'contactEmail']
        anonymous_perms = ['view', 'add']
        authenticated_perms = ['inherit', 'add']

    def __str__(self):
        return self.contactEmail


# ContactTeam is used for other kinds of contact


class ContactReason(Model):
    """Reasons for making contact with the team"""
    name = models.CharField(max_length=255, verbose_name=u"Raison du contact")

    def __str__(self):
        return self.name


class ContactTeam(Model):
    """A submitted contact form"""
    created_at = models.DateTimeField(auto_now_add=True, verbose_name=u"Date d'envoie")
    subject = models.CharField(max_length=255, verbose_name=u"Sujet de l'email")
    contact_reason = models.ForeignKey(ContactReason, verbose_name=u"Raison du contact", null=True,
                                       on_delete=models.SET_NULL)
    content = models.TextField(verbose_name=u"Contenu de l'email")
    contact_email = models.EmailField(max_length=400, verbose_name=u"Adresse email du contact")
    status = models.CharField(max_length=32, default='received', choices=(('received', 'received'),('answered', 'Answered'),('disregard', 'Disregard')), verbose_name="Status")

    class Meta:
        serializer_fields = ['@id', 'created_at','subject','contact_reason', 'content', 'contact_email','status']
        # nested_fields=['contact_reason']
        anonymous_perms = ['view', 'add']
        authenticated_perms = ['inherit', 'add']

    def __str__(self):
        return self.subject


class TextSupporter(Model):
    """Allow citizens to sign a text that is in an action group"""
    created_at = models.DateTimeField(auto_now_add=True)
    fullname = models.CharField(max_length=350, default='nom/prenoms',verbose_name=u"Nom, Prénom")
    email = models.CharField(max_length=350, default='adresse',verbose_name=u"Email",unique=True,error_messages={'unique': u'Quelqu\'un.e a déjà rejoins la plainte avec cette adresse email'})
    
    address = models.CharField(max_length=350, default='adresse',verbose_name=u"Adresse postale complète")
    optin =  models.BooleanField(default=False)
    optin2 =  models.BooleanField(default=False,blank=True, null=True,verbose_name=u"Sign up for Risefor Notifications")

    documentToSign = models.URLField(verbose_name=u"Document a signer")

    linkedSupport = models.ForeignKey(AppelSoutien, default='', verbose_name=u"Signature à l'appel", on_delete=models.CASCADE)

    class Meta:
        serializer_fields = ['@id','fullname','email','address',"optin",'optin2','documentToSign','linkedSupport']
        anonymous_perms = ['add',]
        authenticated_perms = ['inherit', 'add']
        nested_fields = ['linkedSupport']
        ordering=['-created_at']

    def __str__(self):
        return self.fullname


class OrganisationSupporter(Model):
    """Allow citizens to sign a text that is in an action group"""
    created_at = models.DateTimeField(auto_now_add=True)
    organisationName = models.CharField(max_length=350, verbose_name=u"Nom de l'organisation")
    organisationType = models.ForeignKey(HelpOrgType, verbose_name=u"Type de collectif", on_delete=models.CASCADE)
    address = models.CharField(max_length=350, verbose_name=u"Adresse postale complète")
    
    citizenFullName = models.CharField(max_length=350, verbose_name=u"Nom, Prénom")
    citizenPosition = models.CharField(max_length=350, verbose_name=u"Position dans l'organisation")
    email = models.CharField(max_length=350, verbose_name=u"Email",unique=True,error_messages={'unique': u'Quelqu\'un.e a déjà rejoins la plainte avec cette adresse email'})
      
    optin1 =  models.BooleanField(default=False)
    optin2 =  models.BooleanField(default=False)

    documentToSign = models.URLField(verbose_name=u"Document a signer")
    linkedSupport = models.ForeignKey(AppelSoutien, verbose_name=u"Signature à l'appel", on_delete=models.CASCADE)
  
    class Meta:
        serializer_fields = ['@id','organisationName','documentToSign','organisationType','citizenFullName','citizenPosition','email','address',"optin1","optin2",'linkedSupport']
        anonymous_perms = ['add',]
        authenticated_perms = ['inherit', 'add']
        nested_fields = ['linkedSupport']
        ordering=['-created_at']

    def __str__(self):
        return self.organisationName + self.citizenFullName

@receiver(post_save, sender=TextSupporter)
@receiver(post_save, sender=OrganisationSupporter)
def send_email_on_supportsignup(sender, instance, created, **kwargs):
    if created:
        from django.core.mail import EmailMultiAlternatives
        from django.template.loader import get_template
        from django.template import Context, Template

        textTemplate = get_template('email_templates/appel-soutien-confirmation.html')
        htmlTemplate = get_template('email_templates/appel-soutien-confirmation.html')
        

        print("confirmation message", instance.linkedSupport.emailConfirmationContenu)
        
        #Replace backline with <br> to display them in HTML email
        personnalizedContent = instance.linkedSupport.emailConfirmationContenu
        personnalizedContent = personnalizedContent.replace("\n\r","<br>")
        personnalizedContent = personnalizedContent.replace("\r","<br>")
        d = {"support": {
                 "email_campaigner_subject": instance.linkedSupport.emailConfirmationSujet,
                 "email_campaigner_message": personnalizedContent,
             }
            }
        print('linked support is',instance.linkedSupport)

        text_content = textTemplate.render(d['support'])
        html_content = htmlTemplate.render(d['support'])
        ### Content processor replaces the <br> by a string (&lt;br&gt;), 
        #let's make it a br again
        html_content = html_content.replace("&lt;br&gt;","<br>")
        ### and just remove it for text version
        text_content = text_content.replace("&lt;br&gt;","\n")


        email = EmailMultiAlternatives( 
            subject=str(instance.linkedSupport.emailConfirmationSujet),
            body=text_content,
            from_email='"{}" <{}>'.format(settings.INSTANCE_NAME, settings.DEFAULT_FROM_EMAIL),
            reply_to=[instance.linkedSupport.emailConfirmationReplyTo],
            to=[instance.email],
        )
        email.attach_alternative(html_content, "text/html")
        if instance.linkedSupport.documentTelecharger:
            print('./media/'+str(instance.linkedSupport.documentTelecharger))
            email.attach_file('./media/'+str(instance.linkedSupport.documentTelecharger))
        #also add plainte
        email.attach_file('./media/plainte/Recours-CJR-VF.pdf')
        email.send()


class Newsletter(Model):
    created_at = models.DateTimeField(auto_now_add=True)
    fullname = models.CharField(max_length=250, verbose_name=u"Nom, prénom")
    userEmail = models.EmailField(max_length=250, verbose_name=u"Email")
    signedUpFrom = models.CharField(max_length=250, verbose_name=u"Source de l'inscription")
    optin = models.CharField(max_length=15,verbose_name=u"Optin a la newsletter",blank=True,null=True)

    class Meta:
        serializer_fields = ['@id','fullname','userEmail','signedUpFrom','optin']
        anonymous_perms = ['add','view']
        authenticated_perms = ['inherit', 'add','view']
        ordering = ['-created_at']

    def __str__(self):
        return self.fullname

@receiver(post_save, sender=ActionGroup)
def save_autojoin_actiongroup(sender, instance, created, **kwargs):
    if created and instance.author is not None:
        instance.members.add(instance.author)


@receiver(pre_save, sender=ActionGroup)
def actiongroup_generate_slug(sender, instance, **kwargs):
    instance.slug = default_slug = slugify(instance.name)[:40]
    for index in itertools.count(1):
        if not ActionGroup.objects.filter(slug=instance.slug).exists():
            break
        instance.slug = "{}-{}".format(default_slug, index)


@receiver(pre_save, sender=Representative)
def representative_generate_slug(sender, instance, **kwargs):
    instance.slug = default_slug = slugify("{} {}".format(instance.firstname, instance.lastname))[:40]
    for index in itertools.count(1):
        if not Representative.objects.filter(slug=instance.slug).exists():
            break
        instance.slug = "{}-{}".format(default_slug, index)


def checkEmailValidity(currentEmail):
    print("checking email ", currentEmail)
    isEmailValid = ApprovedEmail.objects.filter(name__iexact=currentEmail).exists()
    print("result",isEmailValid)
    return isEmailValid

@receiver(pre_save, sender=ElectedOfficialContact)
def check_valid_email_ElectedOfficialContact(sender, instance, **kwargs):
    targetedEmails = instance.representativeEmails
    allowSending = True
    #check if there are multiple emails
    if "," in targetedEmails:
        targetedEmailArray = targetedEmails.split(",")
        # print("email array is",targetedEmailArray)
        for singleEmail in targetedEmailArray:
            # print("array, testing",singleEmail)
            isValid = checkEmailValidity(singleEmail.strip())
            if isValid == False:
                allowSending = False    
    else:
        isValid = checkEmailValidity(targetedEmails)
        # print("is email allowed?",isValid)  
        if isValid == False:
            allowSending = False

    print("End of tests, are emails allowed ?",allowSending)
    if allowSending == False:
        print("pre save check is false, no saving")
        raise ValueError("Some emails are not valid",targetedEmails)


# Send email to targets
@receiver(post_save, sender=ElectedOfficialContact)
def send_email_on_ElectedOfficialContact(sender, instance, created, **kwargs):
    if created:
        from django.core.mail import EmailMultiAlternatives
        from django.template.loader import get_template
        from django.template import Context, Template

        textTemplate = get_template('email_templates/contact-representative-email.html')
        htmlTemplate = get_template('email_templates/contact-representative-email.html')
        #Replace backline with <br> to display them in HTML email
        personnalizedContent = instance.content
        personnalizedContent = personnalizedContent.replace("\r\n","<br>")
        personnalizedContent = personnalizedContent.replace("\n","<br>")

        # print(personnalizedContent)

        d = {"emailSender": {
                 "citizen_name": instance.userName,
                 'email_content': personnalizedContent,
             }
            }
        ### customize email
        text_content = textTemplate.render(d['emailSender'])
        html_content = htmlTemplate.render(d['emailSender'])
        # print('after context',html_content)

        ### Content processor replaces the <br> by a string (&lt;br&gt;), 
        #let's make it a br again
        html_content = html_content.replace("&lt;br&gt;","<br>")
        ### and just remove it for text version
        text_content = text_content.replace("&lt;br&gt;","\n")

        # From / To info
        userEmail = instance.userEmail
        userSenderEmail = re.sub("@.*","@risefor.org",userEmail)
        print('Contacting rep - sender',userSenderEmail)
        emailTo = instance.representativeEmails
        emailTo = re.sub(' ','',emailTo)
        emailTo = emailTo.split(',')
        emailSubject = str(instance.userName +" | "+ instance.subject)
        #bug - sometimes 'instance.linkedAction' isnt loaded
        if instance.linkedAction:
            #####
            ## If there's a CC to be added
            #####
            emailCC = instance.linkedAction.addCCToEmail
            if emailCC is not None:
                emailCC = emailCC.split(',')
            else:
                emailCC = ''
            ####
            ## If there's a PJ
            ####
            emailPJ = instance.linkedAction.addPJToEmail
            emailPJ_file = ""
            if emailPJ :
                emailPJ_file = emailPJ.path
                # print("PJ in email :",emailPJ_file)
            else:
                emailPJ_file = None
                print("No PJ to be added")
        else:
            emailCC = ''
            emailPJ_file = None

        print("cc's are",emailCC)
        ### build email
        email = EmailMultiAlternatives( 
            subject=emailSubject,
            body=text_content,
            from_email='"{}" <{}>'.format(instance.userName, userSenderEmail),
            to=emailTo,
            reply_to=[userEmail],
            cc=emailCC
        )
        email.attach_alternative(html_content, "text/html")
        if emailPJ_file:
            print('Adding PJ to email',emailPJ_file)
            email.attach_file(emailPJ_file)
        email.send()

#Send Confirmation Email to user
@receiver(post_save, sender=ElectedOfficialContact)
def send_email_confirmation_on_ElectedOfficialContact(sender, instance, created, **kwargs):
    # from django.template.loader import render_to_string
    if created:
        from django.core.mail import EmailMultiAlternatives
        from django.template.loader import get_template
        from django.template import Context, Template
        
        textTemplate = get_template('email_templates/interpellation-confirmation.txt')
        htmlTemplate = get_template('email_templates/interpellation-confirmation.html')
        
        #Replace backline with <br> to display them in HTML email
        personnalizedContent = instance.content
        # personnalizedContent = personnalizedContent.replace("\r","<br>")
        personnalizedContent = personnalizedContent.replace("\r\n","<br>")
        personnalizedContent = personnalizedContent.replace("\n","<br>")

        #bug - sometimes 'instance.linkedAction' isnt loaded
        if instance.linkedAction:
            emailCC = instance.linkedAction.addCCToEmail
            if emailCC is not None:
                emailCC = emailCC.split(',')
            else:
                emailCC = ''

            ####
            ## If there's a PJ
            ####
            emailPJ = instance.linkedAction.addPJToEmail
            emailPJ_file = ""
            if emailPJ :
                emailPJ_file = emailPJ.path
                # print("Confirmation email - PJ in email :",emailPJ_file)
            else:
                emailPJ_file = None
                print("Confirmation email - No PJ to be added")
        else:
            emailCC = ''
            emailPJ_file = None

        print("cc's are",emailCC)

        d = {"emailSender": {
                 "email_destinataires": str(instance.representativeEmails) + emailCC,
                 "email_subject": str(instance.subject),
                 'email_content':personnalizedContent
             }
            }
        # print('context is',Context(d['emailSender']))
        ### Needed vars
        userEmail = instance.userEmail     
        emailSubject = 'Interpellation Risefor | Récapitulatif de votre envoi'
        ### customize email
        text_content = textTemplate.render(d['emailSender'])
        html_content = htmlTemplate.render(d['emailSender'])
        # print('after context',html_content)
        ### Content processor replaces the <br> by a string (&lt;br&gt;), 
        #let's make it a br again
        html_content = html_content.replace("&lt;br&gt;","<br>")
        ### and just remove it for text version
        text_content = text_content.replace("&lt;br&gt;","\n")

        ### build email
        email = EmailMultiAlternatives( 
            subject=emailSubject,
            body=text_content,
            from_email='"{}" <{}>'.format(settings.INSTANCE_NAME, settings.DEFAULT_FROM_EMAIL),
            to=[userEmail],
        )
        email.attach_alternative(html_content, "text/html")
        if emailPJ_file:
            print('Confirmation email - Adding PJ to email',emailPJ_file)
            email.attach_file(emailPJ_file)
        email.send()

#update count of emails sent to in all actions
@receiver(post_save, sender=ElectedOfficialContact)
def save_updateRepresentativeEmailCounter_actiongroup(sender, instance, created, **kwargs):
    if created:
        #in some cases the linkedAction js bugs
        # to avoid issues, don't launch if emptu
        # print('linkedAction',instance.linkedAction)
        try:
            actionGroup = ActionGroup.objects.get(name=instance.linkedAction.name)
            currentCount = actionGroup.representativeEmailSentCounter 
            newCount = (currentCount + 1)
            print("Update counter for ",actionGroup.name,newCount)
            actionGroup.representativeEmailSentCounter  = newCount
            actionGroup.save()

        except Exception as e:
            # Handle the case where the object does not exist
            print("ActionGroup with the specified name does not exist.",e)
            # Additional error handling or logging can be added here
          

# Newsletter subscription - If optin is checked add user to list
@receiver(post_save, sender=ElectedOfficialContact)
@receiver(post_save, sender=TextSupporter)
def save_addToNewsletter_actiongroup(sender, instance, created, **kwargs):
    if created:
        ##Get value of Optin
        senderName = sender.__name__
        if senderName == 'ElectedOfficialContact':
            optinToNewletter = instance.optin2
        elif senderName == 'TextSupporter':
            optinToNewletter = instance.optin2
        print('newsletter choice is',optinToNewletter)

        if optinToNewletter == True:
            from risefor_lobbying.models import Newsletter
            newsletter = Newsletter()
            if senderName == 'ElectedOfficialContact':
                newsletter.fullname = instance.userName
                newsletter.userEmail = instance.userEmail
                newsletter.optin = instance.optin2
                newsletter.signedUpFrom = senderName + ': ' + instance.linkedAction.name
                print('add to newsletter',newsletter.fullname)

            elif senderName == 'TextSupporter':
                newsletter.fullname = instance.fullname
                newsletter.userEmail = instance.email
                newsletter.optin = instance.optin2
                newsletter.signedUpFrom = senderName + ': ' + instance.linkedSupport.name
                print('add to newsletter',newsletter.fullname,newsletter.userEmail,newsletter.optin )


            newsletter.save()

@receiver(post_save, sender=ContactTeam)
def send_email_on_contactteam(sender, instance, created, **kwargs):
    if created:
        body = {
            'subject': instance.subject,
            'contact_reason': instance.contact_reason,
            'content': instance.content,
            'contact_email': instance.contact_email,
        }
        content = {"%s: %s" % (key, value) for (key, value) in body.items()}
        # turn those strings into 1 block of text separated by newlines
        content = "\n".join(content)
        email = EmailMessage(
            "Nouveau Contact " + instance.subject,
            content,
            from_email='community@risefor.org',
            reply_to=[instance.contact_email],
            to=[DEFAULT_FROM_EMAIL]
        )
        email.send()


@receiver(post_save, sender=HelpInstall)
def send_email_on_contact(sender, instance, created, **kwargs):
    if created:
        body = {
            'Nom de organisation': instance.orgname,
            'Type organisation': instance.orgType,
            'Niveau aide nécessaire': instance.helpLevel,
            'Utilisation de outil': instance.usage_tool,
            'Commentaire': instance.extra_msg,
            'Email de contact': instance.contactEmail,

        }
        content = {"%s: %s" % (key, value) for (key, value) in body.items()}
        # turn those strings into 1 block of text separated by newlines
        content = "\n".join(content)
        email = EmailMessage(
            "Demande d'aide par " + instance.orgname,
            content,
            to=[DEFAULT_FROM_EMAIL]
        )
        email.send()

