#from typing import OrderedDict
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.serializers import ValidationError
from rest_framework.response import Response

from djangoldp.views import LDPViewSet
from risefor_lobbying.models import ElectedOfficialContact,ConsultationPublic,ActionGroup,financeCampaign, Representative
from datetime import date,datetime
from django.conf import settings

import re

class RepresentativeViewSet(LDPViewSet):
     model=Representative

     def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).exclude(function__icontains="attache")

class RepresentativeSearchViewSet(LDPViewSet):
    model=Representative
    fields = ['@id','fullname','photo','politicalParty','function','allPostalCodes','directEmails','twitterAccount']


class financeCampaignViewSet(LDPViewSet):
    model=financeCampaign
    def get_queryset(self, *args, **kwargs):
        querySet = super().get_queryset(*args, **kwargs)
        localActiveCampaign = querySet.filter(status__contains='active')
        # print("local actions are",localActiveCampaign)
        return localActiveCampaign

class ActionViewSet(LDPViewSet):
    model=ActionGroup
    def get_queryset(self, *args, **kwargs):
        querySet = super().get_queryset(*args, **kwargs)
        localActiveActions = querySet.filter(status__contains='active')
        print("local actions are",localActiveActions)
        return localActiveActions

class ConsultationViewSet(LDPViewSet):
    model=ConsultationPublic
    def get_queryset(self, *args, **kwargs):
        querySet = super().get_queryset(*args, **kwargs)
        localActiveConsultation = querySet.filter(status__contains='active')
        return localActiveConsultation

class passedActionViewSet(LDPViewSet):
     model=ActionGroup
     def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).filter(status__contains='finished')


class passedConsultationViewSet(LDPViewSet):
     model=ConsultationPublic
     def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).filter(status__contains='finished')



class ConvergenceViewSet(LDPViewSet):
    '''
    ViewSet using Actions between two parameterised dates. If only one date is passed then
    only one is applied, i.e. if you pass start_date without end_date it will return all
    Actions after the start_date
    '''
    model=ActionGroup

    def get_queryset(self, *args, **kwargs):
        start_date = self.request.query_params.get('start_date')
        end_date = self.request.query_params.get('end_date')

        queryset = super().get_queryset(*args, **kwargs)

        if start_date is not None:
            try:
                start_date = datetime.strptime(start_date, '%Y-%m-%d')
            except ValueError:
                raise ValidationError('start_date was in an invalid format. Should be YYYY-MM-DD')
            queryset = queryset.filter(actiondate__gt=start_date)

        if end_date is not None:
            try:
                end_date = datetime.strptime(end_date, '%Y-%m-%d')
            except ValueError:
                raise ValidationError('end_date was in an invalid format. Should be YYYY-MM-DD')
            queryset = queryset.filter(actiondate__lt=end_date)

        return queryset

#view to display total emails
class TotalSubmission(LDPViewSet):
    model = ActionGroup
    def get_queryset(self, *args, **kwargs):
        emailArray = {}
        for action in ActionGroup.objects.all():
            # print('action is',action)
            emailArray[action.name] = action.representativeEmailSentCounter
        return emailArray

    def list(self, request, *args, **kwargs):
        return Response({'Emails envoyé par campagne': self.get_queryset()}) 

def shortLinkRedirect(request, slugs):
    from risefor_lobbying.models import UrlShortner
    from django.shortcuts import redirect
    print("Short like redirect - requested slug",slugs,settings.INSTANCE_DOMAINE_NAME+'/s/'+slugs)
    
    # print("shorlink result",UrlShortner.objects.get(shortLink=settings.INSTANCE_DOMAINE_NAME+'/s/'+slugs) )
    #search for matching url in shortners

    try:
        data = UrlShortner.objects.get(shortLink=settings.INSTANCE_DOMAINE_NAME+'/s/'+slugs)
        frontUrl = data.url.replace(":","%3A")
        # print("data url",data,'for action name',data.actionName)

        ### Handle "consultation/enquete" section
        if "Lien pour la consultation" in data.actionName:
        #take URL and format it to front end Urls
            finalUrl =  "/consultations-publique/consultation-detail/@"+frontUrl +"@"
        
        elif "Lien pour l'appel " in data.actionName:
            #take URL and format it to front end Urls
                finalUrl =  "/appel-soutiens/appel-soutiens-detail/@"+frontUrl +"@"
        else:
            finalUrl =  "/actions/action-group-detail/@"+frontUrl +"@"
        print("final url",finalUrl)
        #redirect
        return redirect(finalUrl)
    #if no match, redirect to actions listing
    except (UrlShortner.DoesNotExist) or None:
        print("Short link redirect - except",UrlShortner)
        return redirect("/actions")


#########
### Twitter Counter
## https://git.risefor.org/applications/risefor-front/-/issues/113
## Evolution needed https://git.risefor.org/applications/risefor-front/-/issues/144
#########
from django.http import JsonResponse
from django.views.decorators.http import require_POST
import json
from server.settings import *

@require_POST
def increment_twitter_click_counter(request):
    try:
        # Load JSON data from the request body
        data = json.loads(request.body)
        actionUrl = data.get('currentUrl')

        # Get the current domain
        currentDomain = settings.INSTANCE_DOMAINE_NAME.replace('http://','').replace('https://','')
        #####
        ## Federation - Improvement needed https://git.risefor.org/applications/risefor-front/-/issues/144
        ## As a V1, exclude clics from other instance
        #####
        ## Instance host is present after the action-group-detail/ in url
        # Extract the host from the actionUrl
        actionUrlHost = re.search('action-group-detail\/.*', actionUrl).group()
        actionUrlHost = actionUrlHost.replace('%3A',':')
        print('url host search', actionUrlHost, currentDomain) ## for testing phase
        if currentDomain not in actionUrlHost:
            print('Tweet on action from other instance, evolution needed')
            return JsonResponse({'status': 'error'})
        
        # Extract the action ID from the URL
        searchForID = re.search(r'(/actiongroups/\d+)', actionUrl)
        if not searchForID:
            print('No action group in url given, error')
            return JsonResponse({'status': 'error'})

        # Extract the number from the matched string
        actionId = int(re.search('\d+', searchForID.group(1)).group())
        # print('actionId is', actionId)  

        # Get the ActionGroup instance
        instance = ActionGroup.objects.get(pk=actionId)

        # Increment the twitterClickCounter field
        instance.twitterClickCounter += 1
        print('updating twitter counter for', instance.name,instance.twitterClickCounter)
        instance.save()

        # Return a JSON response indicating success
        return JsonResponse({'status': 'success'})
        
    except (json.JSONDecodeError, AttributeError, ValueError, ActionGroup.DoesNotExist):
        return JsonResponse({'status': 'error', 'message': 'Invalid JSON data or action not found'}, status=400)